import logging
import logging.config
import yaml
import importlib
import importlib.metadata
import os
import re
from typing import Dict, Any
from pyfiglet import Figlet
from dotenv import load_dotenv


def read_config_file(file_path):
    try:
        # 打开 YAML 文件
        with open(file_path, "r", encoding="utf-8") as file:
            # 读取文件内容并展开环境变量
            yaml_str = file.read()
            expanded_yaml = os.path.expandvars(yaml_str)
            
            # 检查是否有未替换的环境变量
            unresolved = re.findall(r'\$\{([^}]+)\}', expanded_yaml)
            if unresolved:
                print(f"Warning: 以下环境变量未设置: {', '.join(unresolved)}")
            
            # 使用 yaml.safe_load 方法解析 YAML 文件内容
            data = yaml.safe_load(expanded_yaml)
            return data
    except FileNotFoundError:
        raise Exception(f"文件 {file_path} 未找到。")
    except yaml.YAMLError as e:
        raise Exception(f"解析 {file_path} 文件时出错: {e}")


def initialize_logger(config: Dict[str, Any]) -> logging.Logger:
    """初始化日志配置并返回logger实例"""
    logging.config.dictConfig(config["Logger"])
    return logging.getLogger("openfund-taker")


def create_strategy_instance(
    taker_name: str, configs: Dict[str, Any], logger: logging.Logger, exchangeKey: str
):
    """创建策略实例"""
    module = importlib.import_module(f"taker.{taker_name}")
    strategy_class = getattr(module, taker_name)
    return strategy_class(
        configs, configs["platform"][exchangeKey], configs["common"], logger=logger
    )


def main():
    # 加载 .env 文件 - 尝试多个位置
    from pathlib import Path
    
    # 尝试的路径顺序：当前目录 -> 脚本目录 -> 父目录
    env_paths = [
        Path.cwd() / ".env",
        Path(__file__).parent.parent.parent.parent / ".env",  # openfund-maker/.env
        Path.cwd().parent / ".env",
    ]
    
    env_loaded = False
    for env_path in env_paths:
        if env_path.exists():
            load_dotenv(env_path)
            print(f"已加载环境变量文件: {env_path}")
            env_loaded = True
            break
    
    if not env_loaded:
        load_dotenv()  # 尝试默认加载
        
    print(f"OPENFUND_OKX_PASSWORD: {os.getenv('OPENFUND_OKX_PASSWORD', 'NOT SET')}")

    # 获取版本信息和包名
    version = importlib.metadata.version("openfund-taker")
    package_name = __package__ or "taker"

    # 读取配置文件
    config_path = Path("taker_config.yaml")
    config_data = read_config_file(config_path)

    # 获取配置参数
    common_config = config_data["common"]
    taker_name = common_config.get("actived_taker", "StrategyTaker")

    # 初始化日志
    logger = initialize_logger(config_data)

    # 显示启动信息
    f = Figlet(font="standard")
    logger.info(f"\n{f.renderText('OpenFund Taker')}")
    logger.info(f" ++ {package_name}.{taker_name}:{version} is doing...")
    exchangeKey = common_config.get("exchange_key", "okx")
    # 创建并运行策略实例
    bot = create_strategy_instance(taker_name, config_data, logger, exchangeKey)
    bot.monitor_total_profit()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from datetime import datetime
from decimal import Decimal
from threading import Lock
from typing import Dict, Optional, List
import time


@dataclass
class StopLossOrder:
    """止损订单数据类"""
    order_id: str
    symbol: str
    price: Decimal
    side: str  # 'long' or 'short'
    created_at: datetime
    is_active: bool = True


class DualStopLossManager:
    """
    双止损管理器
    
    管理每个交易对的双止损订单,确保在更新止损时始终保持保护。
    通过先创建新止损订单,成功后再取消旧订单的方式,消除保护间隙。
    """
    
    def __init__(self, exchange, logger, config: dict = None):
        """
        初始化双止损管理器
        
        Args:
            exchange: 交易所接口实例
            logger: 日志记录器
            config: 配置字典,包含:
                - enabled: 是否启用双止损 (默认: False)
                - max_retries: 最大重试次数 (默认: 3)
                - retry_delays: 重试延迟列表 (默认: [1, 2, 4])
                - webhook_url: 通知webhook地址
                - notify_on_failure: 失败时是否通知 (默认: True)
                - notify_on_cleanup_error: 清理错误时是否通知 (默认: True)
        """
        self.exchange = exchange
        self.logger = logger
        
        # 配置管理
        self.config = config or {}
        self.enabled = self.config.get('enabled', False)
        self.max_retries = self.config.get('max_retries', 3)
        self.retry_delays = self.config.get('retry_delays', [1, 2, 4])
        self.webhook_url = self.config.get('webhook_url', '')
        self.notify_on_failure = self.config.get('notify_on_failure', True)
        self.notify_on_cleanup_error = self.config.get('notify_on_cleanup_error', True)
        
        # 活跃止损订单跟踪: {symbol: [StopLossOrder, ...]}
        self._active_orders: Dict[str, List[StopLossOrder]] = {}
        
        # 并发控制锁: {symbol: Lock}
        self._locks: Dict[str, Lock] = {}
        
        self.logger.info(f"[DualSL] 双止损管理器初始化完成 (enabled={self.enabled})")
    
    def _get_lock(self, symbol: str) -> Lock:
        """获取交易对的锁"""
        if symbol not in self._locks:
            self._locks[symbol] = Lock()
        return self._locks[symbol]
    
    def _get_active_orders(self, symbol: str) -> List[StopLossOrder]:
        """获取交易对的活跃止损订单列表"""
        try:
            params = {"ordType": "conditional"}
            orders = self.exchange.fetch_open_orders(symbol=symbol, params=params)
            
            result = []
            for order in orders:
                info = order.get('info', {})
                trigger_price = order.get('stopLossPrice') or info.get('slTriggerPx')
                
                if not trigger_price or trigger_price == '':
                    continue
                
                result.append(StopLossOrder(
                    order_id=order['id'],
                    symbol=symbol,
                    price=Decimal(str(trigger_price)),
                    side=order.get('side', 'unknown'),
                    created_at=datetime.now(),
                    is_active=True
                ))
            
            return result
        except Exception as e:
            self.logger.warning(f"[DualSL] {symbol}: 查询活跃订单失败: {e}")
            return []
    
    def _add_order(self, symbol: str, order: StopLossOrder):
        """添加止损订单到跟踪列表"""
        orders = self._get_active_orders(symbol)
        orders.append(order)
        # 按创建时间排序,最新的在前
        orders.sort(key=lambda x: x.created_at, reverse=True)
        
        # 最多保留2个订单
        if len(orders) > 2:
            self.logger.warning(f"[DualSL] {symbol}: 订单数量超过2个,保留最新的2个")
            self._active_orders[symbol] = orders[:2]
    
    def _remove_order(self, symbol: str, order_id: str):
        """从跟踪列表中移除止损订单"""
        orders = self._get_active_orders(symbol)
        self._active_orders[symbol] = [o for o in orders if o.order_id != order_id]
    
    def get_order_count(self, symbol: str) -> int:
        """获取交易对的活跃止损订单数量"""
        return len(self._get_active_orders(symbol))
    
    def get_latest_order(self, symbol: str) -> Optional[StopLossOrder]:
        """获取交易对的最新止损订单"""
        orders = self._get_active_orders(symbol)
        return orders[0] if orders else None
    
    def clear_orders(self, symbol: str):
        """清除交易对的所有订单跟踪记录"""
        if symbol in self._active_orders:
            del self._active_orders[symbol]
        if symbol in self._locks:
            del self._locks[symbol]

    def _sync_orders_from_exchange(self, symbol: str):
        """从交易所同步已存在的止损订单到内存"""
        if self._get_active_orders(symbol):
            return  # 已有记录，跳过同步
        
        try:
            params = {"ordType": "conditional"}
            orders = self.exchange.fetch_open_orders(symbol=symbol, params=params)
            
            for order in orders:
                # 获取止损价格 (OKX: slTriggerPx)
                info = order.get('info', {})
                trigger_price = (
                    order.get('stopLossPrice') or 
                    info.get('slTriggerPx')
                )
                
                # 跳过止盈订单和无止损价格的订单
                if not trigger_price or trigger_price == '':
                    continue
                    
                sl_order = StopLossOrder(
                    order_id=order['id'],
                    symbol=symbol,
                    price=Decimal(str(trigger_price)),
                    side=order.get('side', 'unknown'),
                    created_at=datetime.now(),
                    is_active=True
                )
                self._add_order(symbol, sl_order)
                self.logger.debug(f"[DualSL] {symbol}: 同步止损订单 id={order['id']} price={trigger_price}")
            
            count = len(self._active_orders.get(symbol, []))
            if count > 0:
                self.logger.info(f"[DualSL] {symbol}: 从交易所同步了 {count} 个止损订单")
        except Exception as e:
            self.logger.warning(f"[DualSL] {symbol}: 同步交易所订单失败: {e}")

    def update_stop_loss(
        self, 
        symbol: str, 
        position: dict, 
        new_sl_price: Decimal,
        order_type: str = 'conditional'
    ) -> bool:
        """
        更新止损订单 (双止损策略)
        
        流程:
        1. 先创建新的止损订单
        2. 如果创建成功,取消最旧的止损订单 (如果已有2个订单)
        3. 如果创建失败,保留现有订单不变
        
        Args:
            symbol: 交易对
            position: 持仓信息
            new_sl_price: 新的止损价格
            order_type: 订单类型 ('conditional', 'market', 'limit')
            
        Returns:
            bool: 是否成功更新止损
        """
        lock = self._get_lock(symbol)
        
        with lock:
            precision = self._get_precision_length(symbol)
            side = position['side']
            
            orders = self._get_active_orders(symbol)
            
            # 检查是否与最新订单价格相同
            if orders and orders[0].price == new_sl_price:
                self.logger.debug(
                    f"[DualSL] {symbol}: 止损价格未变化 {new_sl_price:.{precision}f}, 跳过更新"
                )
                return True
            
            # 第一次初始化：创建两条止损线
            if len(orders) == 0:
                entry_price = Decimal(str(position.get('entryPrice', 0)))
                all_stop_loss_pct = Decimal(str(self.config.get('all_stop_loss_pct', 1))) / Decimal('100')
                
                # 计算第一条止损线：基于all_stop_loss_pct
                if side == 'long':
                    first_sl_price = entry_price * (Decimal('1') - all_stop_loss_pct)
                else:
                    first_sl_price = entry_price * (Decimal('1') + all_stop_loss_pct)
                
                # 计算第二条止损线：在第一条基础上偏移1%
                if side == 'long':
                    second_sl_price = first_sl_price * Decimal('0.99')
                else:
                    second_sl_price = first_sl_price * Decimal('1.01')
                
                self.logger.info(
                    f"[DualSL] {symbol}: 初始化双止损 [{side}] 开仓价={entry_price:.{precision}f} "
                    f"第一条={first_sl_price:.{precision}f} 第二条={second_sl_price:.{precision}f}"
                )
                
                # 创建第一条止损线
                order_id_1 = self._create_stop_loss_order(symbol, position, first_sl_price, order_type)
                if not order_id_1:
                    self.logger.error(f"[DualSL] {symbol}: 创建第一条止损线失败")
                    return False
                
                self.logger.info(
                    f"[DualSL] {symbol}: 第一条止损线创建成功 order_id={order_id_1} price={first_sl_price:.{precision}f}"
                )
                
                # 创建第二条止损线
                order_id_2 = self._create_stop_loss_order(symbol, position, second_sl_price, order_type)
                if not order_id_2:
                    self.logger.warning(f"[DualSL] {symbol}: 创建第二条止损线失败")
                else:
                    self.logger.info(
                        f"[DualSL] {symbol}: 第二条止损线创建成功 order_id={order_id_2} price={second_sl_price:.{precision}f}"
                    )
                
                return True
            
            # 后续更新：创建新止损线
            self.logger.info(
                f"[DualSL] {symbol}: 更新止损 [{side}] "
                f"新价格={new_sl_price:.{precision}f} "
                f"当前订单数={len(orders)}"
            )
            
            new_order_id = self._create_stop_loss_order(symbol, position, new_sl_price, order_type)
            
            if not new_order_id:
                self.logger.error(f"[DualSL] {symbol}: 创建新止损订单失败")
                return False
            
            self.logger.info(
                f"[DualSL] {symbol}: 新止损订单创建成功 order_id={new_order_id} price={new_sl_price:.{precision}f}"
            )
            
            # 如果超过2个订单，取消最旧的
            orders = self._get_active_orders(symbol)
            if len(orders) > 2:
                oldest_order = orders[-1]
                self.logger.info(
                    f"[DualSL] {symbol}: 订单数={len(orders)}, 取消最旧订单 order_id={oldest_order.order_id}"
                )
                
                if self._cancel_stop_loss_order(symbol, oldest_order.order_id):
                    self.logger.info(f"[DualSL] {symbol}: 最旧订单已取消 order_id={oldest_order.order_id}")
                else:
                    self.logger.warning(f"[DualSL] {symbol}: 取消最旧订单失败 order_id={oldest_order.order_id}")
            
            self.logger.info(f"[DualSL] {symbol}: 止损更新完成, 当前订单数={len(self._get_active_orders(symbol))}")
            
            return True
    
    def _get_precision_length(self, symbol: str) -> int:
        """获取价格精度长度"""
        try:
            tick_size = self.exchange.get_tick_size(symbol)
            tick_str = f"{tick_size:.15f}".rstrip('0')
            if '.' in tick_str:
                return len(tick_str.split('.')[1])
            return 0
        except Exception as e:
            self.logger.warning(f"[DualSL] {symbol}: 获取精度失败: {e}, 使用默认精度8")
            return 8

    def _create_stop_loss_order(
        self,
        symbol: str,
        position: dict,
        sl_price: Decimal,
        order_type: str = 'conditional'
    ) -> Optional[str]:
        """
        创建止损订单 (带重试机制)
        
        使用指数退避重试策略:
        - 第1次重试: 延迟1秒
        - 第2次重试: 延迟2秒
        - 第3次重试: 延迟4秒
        
        Args:
            symbol: 交易对
            position: 持仓信息
            sl_price: 止损价格
            order_type: 订单类型
            
        Returns:
            Optional[str]: 订单ID,失败返回None
        """
        precision = self._get_precision_length(symbol)
        amount = abs(Decimal(str(position['contracts'])))
        
        if amount <= 0:
            self.logger.warning(f"[DualSL] {symbol}: 持仓数量为0, 无法创建止损订单")
            return None
        
        side = position['side']
        order_side = 'buy' if side == 'short' else 'sell'
        adjusted_price = f"{sl_price:.{precision}f}"
        
        # 构建止损订单参数
        sl_params = {
            'slTriggerPx': adjusted_price,
            'slOrdPx': '-1',  # 市价止损
            'slTriggerPxType': 'last',
            'tdMode': position['marginMode'],
            'sz': str(amount),
            'cxlOnClosePos': True,
            'reduceOnly': True
        }
        
        if order_type == 'limit':
            sl_params['slOrdPx'] = adjusted_price
        
        # 重试逻辑
        for attempt in range(self.max_retries):
            try:
                self.logger.debug(
                    f"[DualSL] {symbol}: 尝试创建止损订单 (尝试 {attempt + 1}/{self.max_retries}) "
                    f"{order_side} @ {adjusted_price}"
                )
                
                order = self.exchange.exchange.create_order(
                    symbol=symbol,
                    type=order_type,
                    price=sl_price,
                    side=order_side,
                    amount=amount,
                    params=sl_params
                )
                
                self.logger.debug(f"[DualSL] {symbol}: 订单响应: {order}")
                
                order_id = order.get('id') or order.get('info', {}).get('algoId')
                
                if order_id:
                    self.logger.info(
                        f"[DualSL] {symbol}: 止损订单创建成功 "
                        f"order_id={order_id} price={adjusted_price}"
                    )
                    return order_id
                else:
                    self.logger.error(
                        f"[DualSL] {symbol}: 订单响应中未找到订单ID: {order}"
                    )
                    
            except Exception as e:
                error_type = type(e).__name__
                self.logger.warning(
                    f"[DualSL] {symbol}: 创建止损订单失败 (尝试 {attempt + 1}/{self.max_retries}) "
                    f"错误类型={error_type} 错误={str(e)}"
                )
                
                # 如果不是最后一次尝试,等待后重试
                if attempt < self.max_retries - 1:
                    delay = self.retry_delays[attempt] if attempt < len(self.retry_delays) else self.retry_delays[-1]
                    self.logger.info(f"[DualSL] {symbol}: 等待 {delay} 秒后重试...")
                    time.sleep(delay)
                else:
                    # 最后一次尝试失败,发送通知
                    self.logger.error(
                        f"[DualSL] {symbol}: 创建止损订单失败,已达最大重试次数 "
                        f"price={adjusted_price}"
                    )
                    if self.notify_on_failure:
                        self._notify_stop_loss_failure(symbol, sl_price, str(e))
        
        return None
    
    def _cancel_stop_loss_order(self, symbol: str, order_id: str) -> bool:
        """
        取消止损订单
        
        Args:
            symbol: 交易对
            order_id: 订单ID
            
        Returns:
            bool: 是否成功取消
        """
        try:
            self.logger.debug(f"[DualSL] {symbol}: 取消止损订单 order_id={order_id}")
            
            params = {
                "algoId": [order_id],
                "trigger": 'trigger'
            }
            
            result = self.exchange.exchange.cancel_orders(
                ids=[order_id],
                symbol=symbol,
                params=params
            )
            
            if result and len(result) > 0:
                self.logger.info(f"[DualSL] {symbol}: 订单取消成功 order_id={order_id}")
                return True
            else:
                self.logger.warning(
                    f"[DualSL] {symbol}: 订单取消响应为空 order_id={order_id}"
                )
                return False
                
        except Exception as e:
            self.logger.error(
                f"[DualSL] {symbol}: 取消订单失败 order_id={order_id} 错误={str(e)}"
            )
            return False

    def _notify_stop_loss_failure(self, symbol: str, sl_price: Decimal, error: str):
        """
        通知止损创建失败
        
        Args:
            symbol: 交易对
            sl_price: 止损价格
            error: 错误信息
        """
        if not self.webhook_url:
            return
        
        try:
            import requests
            
            precision = self._get_precision_length(symbol)
            message = (
                f"[DualSL] 止损创建失败\n"
                f"交易对: {symbol}\n"
                f"止损价格: {sl_price:.{precision}f}\n"
                f"错误: {error}\n"
                f"时间: {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}"
            )
            
            payload = {
                "msg_type": "text",
                "content": {"text": message}
            }
            
            response = requests.post(
                self.webhook_url,
                json=payload,
                headers={'Content-Type': 'application/json'},
                timeout=5
            )
            
            if response.status_code == 200:
                self.logger.debug(f"[DualSL] {symbol}: 失败通知已发送")
            else:
                self.logger.warning(
                    f"[DualSL] {symbol}: 发送通知失败 status={response.status_code}"
                )
                
        except Exception as e:
            self.logger.warning(f"[DualSL] {symbol}: 发送通知异常: {e}")
    
    def _notify_cleanup_error(self, symbol: str, error: str):
        """
        通知清理错误
        
        Args:
            symbol: 交易对
            error: 错误信息
        """
        if not self.webhook_url:
            return
        
        try:
            import requests
            
            message = (
                f"[DualSL] 持仓清理错误\n"
                f"交易对: {symbol}\n"
                f"错误: {error}\n"
                f"时间: {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}"
            )
            
            payload = {
                "msg_type": "text",
                "content": {"text": message}
            }
            
            response = requests.post(
                self.webhook_url,
                json=payload,
                headers={'Content-Type': 'application/json'},
                timeout=5
            )
            
            if response.status_code == 200:
                self.logger.debug(f"[DualSL] {symbol}: 清理错误通知已发送")
            else:
                self.logger.warning(
                    f"[DualSL] {symbol}: 发送通知失败 status={response.status_code}"
                )
                
        except Exception as e:
            self.logger.warning(f"[DualSL] {symbol}: 发送通知异常: {e}")

    def cleanup_position(self, symbol: str) -> bool:
        """
        清理已平仓持仓的所有止损订单
        
        当持仓被平仓时调用此方法,取消所有相关的止损订单并清理内部跟踪记录。
        
        Args:
            symbol: 交易对
            
        Returns:
            bool: 是否成功清理
        """
        lock = self._get_lock(symbol)
        
        with lock:
            orders = self._get_active_orders(symbol)
            
            if not orders:
                self.logger.debug(f"[DualSL] {symbol}: 没有需要清理的止损订单")
                return True
            
            self.logger.info(
                f"[DualSL] {symbol}: 开始清理持仓, 订单数={len(orders)}"
            )
            
            success = True
            cancelled_count = 0
            
            # 取消所有止损订单
            for order in orders:
                if self._cancel_stop_loss_order(symbol, order.order_id):
                    cancelled_count += 1
                else:
                    success = False
                    self.logger.warning(
                        f"[DualSL] {symbol}: 取消订单失败 order_id={order.order_id}"
                    )
            
            # 验证订单是否已取消
            if not self._verify_orders_cancelled(symbol, [o.order_id for o in orders]):
                success = False
                error_msg = f"部分订单未能成功取消"
                self.logger.error(f"[DualSL] {symbol}: {error_msg}")
                
                if self.notify_on_cleanup_error:
                    self._notify_cleanup_error(symbol, error_msg)
            
            # 清理内部跟踪记录
            self.clear_orders(symbol)
            
            self.logger.info(
                f"[DualSL] {symbol}: 清理完成, 已取消={cancelled_count}/{len(orders)} "
                f"成功={success}"
            )
            
            return success
    
    def _verify_orders_cancelled(self, symbol: str, order_ids: List[str]) -> bool:
        """
        验证订单是否已取消
        
        Args:
            symbol: 交易对
            order_ids: 订单ID列表
            
        Returns:
            bool: 所有订单是否都已取消
        """
        try:
            # 获取当前的止损订单
            params = {
                "ordType": "conditional",
            }
            
            open_orders = self.exchange.exchange.fetch_open_orders(symbol=symbol, params=params)
            
            # 检查是否还有未取消的订单
            open_order_ids = set()
            for order in open_orders:
                order_id = order.get('id') or order.get('info', {}).get('algoId')
                if order_id:
                    open_order_ids.add(order_id)
            
            # 检查我们要取消的订单是否还在
            remaining = set(order_ids) & open_order_ids
            
            if remaining:
                self.logger.warning(
                    f"[DualSL] {symbol}: 以下订单仍然存在: {remaining}"
                )
                return False
            
            self.logger.debug(f"[DualSL] {symbol}: 所有订单已确认取消")
            return True
            
        except Exception as e:
            self.logger.error(
                f"[DualSL] {symbol}: 验证订单取消状态失败: {e}"
            )
            return False



# -*- coding: utf-8 -*-
from typing import override
from cachetools import TTLCache
from core.smc.TF import TF
from core.smc.AdaptiveTargetFinder import AdaptiveTargetFinder
from core.utils.OPTools import OPTools
from taker.StrategyTaker import StrategyTaker


class LiquidityGrabStrategyTaker(StrategyTaker):
    """
    流动性抓取形态策略Taker端
    负责基于流动性目标的止盈止损管理
    """

    def __init__(
        self, g_config, platform_config, common_config, logger=None, exchangeKey="okx"
    ) -> None:
        super().__init__(
            g_config=g_config,
            platform_config=platform_config,
            common_config=common_config,
            logger=logger,
            exchangeKey=exchangeKey,
        )

        # 流动性抓取特有的状态追踪
        self.has_init_SL_TPs = {}  # 是否已初始化止盈止损
        self.liquidity_targets = {}  # 流动性目标追踪
        self.order_block_levels = {}  # 订单块水平追踪
        self.entering_liquidity_tps = {}  # 进入流动性止盈监控

        cache_ttl = common_config.get("cache_ttl", 60)
        self.htf_cache = TTLCache(maxsize=100, ttl=int(cache_ttl * 60))
        self.atf_cache = TTLCache(maxsize=100, ttl=int(cache_ttl * 60))
        
        # 初始化自适应目标位查找器
        # 使用全局的 all_stop_loss_pct 和 all_TP_SL_ratio 作为 min_profit_percent 和 min_profit_ratio
        strategy_config = g_config.get("strategy", {})
        adaptive_config = strategy_config.get("adaptive_target", {}).copy()
        
        # 从全局配置中获取止损和盈亏比参数
        all_stop_loss_pct = strategy_config.get("all_stop_loss_pct", 2.0)
        all_TP_SL_ratio = strategy_config.get("all_TP_SL_ratio", 1.5)
        
        # 设置 min_profit_percent 和 min_profit_ratio（如果配置中没有指定）
        if "min_profit_percent" not in adaptive_config:
            adaptive_config["min_profit_percent"] = all_stop_loss_pct
        if "min_profit_ratio" not in adaptive_config:
            adaptive_config["min_profit_ratio"] = all_TP_SL_ratio
        
        # 更新策略配置
        strategy_config_with_adaptive = strategy_config.copy()
        strategy_config_with_adaptive["adaptive_target"] = adaptive_config
        
        self.adaptive_target_finder = AdaptiveTargetFinder(
            strategy_maker=self, 
            config=strategy_config_with_adaptive
        )

    @override
    def reset_SL_TP(self, symbol=None, attachType="BOTH"):
        """重置止盈止损状态"""
        super().reset_SL_TP(symbol, attachType)
        if not symbol:
            self.has_init_SL_TPs.clear()
            self.liquidity_targets.clear()
            self.order_block_levels.clear()
            self.entering_liquidity_tps.clear()
        elif attachType == self.BOTH_KEY and symbol in self.has_init_SL_TPs:
            self.has_init_SL_TPs.pop(symbol, None)
            self.liquidity_targets.pop(symbol, None)
            self.order_block_levels.pop(symbol, None)
            self.entering_liquidity_tps.pop(symbol, None)

    def init_liquidity_grab_SL_TP(
        self, symbol: str, position, tfs: dict, strategy: dict
    ) -> bool:
        """
        基于流动性抓取策略的止盈止损初始化
        止损：基于ATF订单块边界
        止盈：基于等高/等低流动性位置
        """
        open_body_break = bool(strategy.get("open_body_break", True))
        stop_loss_buffer_ticks = int(strategy.get("stop_loss_buffer_ticks", 2))
        min_profit_ratio = OPTools.ensure_decimal(strategy.get("min_profit_ratio", 1.5))

        precision = self.get_precision_length(symbol)
        pos_side = position[self.SIDE_KEY]
        side = self.SELL_SIDE if pos_side == self.SHORT_KEY else self.BUY_SIDE

        htf = tfs[self.HTF_KEY]
        atf = tfs[self.ATF_KEY]
        etf = tfs[self.ETF_KEY]

        # 1. 获取ATF数据分析支撑阻力位和订单块
        atf_df = self.get_historical_klines_df(symbol=symbol, tf=atf)
        atf_struct = self.build_struct(
            symbol=symbol, data=atf_df, is_struct_body_break=open_body_break
        )
        atf_OBs_df = self.find_OBs(
            symbol=symbol, struct=atf_struct, is_struct_body_break=open_body_break
        )

        atf_support_resistance = self.get_support_resistance_from_OBs(
            symbol=symbol, obs_df=atf_OBs_df, struct_df=atf_struct
        )

        if atf_support_resistance is None:
            self.logger.info(
                f"{symbol} : ATF {atf} 未找到支撑阻力位，使用默认百分比止盈止损"
            )
            return self._init_default_SL_TP(symbol, position, strategy)

        # 2. 识别等高等低流动性目标
        liquidity_target = self._identify_liquidity_target(
            symbol, atf_df, atf_struct, pos_side, strategy
        )

        if not liquidity_target:
            self.logger.info(f"{symbol} : ATF {atf} 未找到流动性目标，使用支撑阻力位")
            return self._init_support_resistance_SL_TP(
                symbol, position, atf_support_resistance, stop_loss_buffer_ticks
            )

        # 3. 计算基于流动性的止盈止损
        tick_size = self.get_tick_size(symbol)
        price_offset = stop_loss_buffer_ticks * tick_size

        atf_support_OB = atf_support_resistance.get(self.SUPPORT_OB_KEY)
        atf_resistance_OB = atf_support_resistance.get(self.RESISTANCE_OB_KEY)

        if pos_side == self.LONG_KEY:
            # 多头：止损=支撑位下方订单块底部-缓冲，止盈=等高流动性
            if atf_support_OB:
                sl_price = self.toDecimal(
                    atf_support_OB[self.OB_LOW_COL]
                ) - self.toDecimal(price_offset)
                self.order_block_levels[symbol] = atf_support_OB[self.OB_LOW_COL]
            else:
                sl_price = self.toDecimal(
                    atf_support_resistance[self.SUPPORT_PRICE_KEY]
                ) - self.toDecimal(price_offset)

            tp_price = self.toDecimal(liquidity_target["price"]) + self.toDecimal(
                price_offset
            )

            # 设置进入流动性监控的触发价格
            if atf_resistance_OB:
                entering_trigger_price = atf_resistance_OB[self.OB_HIGH_COL]
            else:
                entering_trigger_price = atf_support_resistance[
                    self.RESISTANCE_PRICE_KEY
                ]

        else:
            # 空头：止损=阻力位上方订单块顶部+缓冲，止盈=等低流动性
            if atf_resistance_OB:
                sl_price = self.toDecimal(
                    atf_resistance_OB[self.OB_HIGH_COL]
                ) + self.toDecimal(price_offset)
                self.order_block_levels[symbol] = atf_resistance_OB[self.OB_HIGH_COL]
            else:
                sl_price = self.toDecimal(
                    atf_support_resistance[self.RESISTANCE_PRICE_KEY]
                ) + self.toDecimal(price_offset)

            tp_price = self.toDecimal(liquidity_target["price"]) - self.toDecimal(
                price_offset
            )

            # 设置进入流动性监控的触发价格
            if atf_support_OB:
                entering_trigger_price = atf_support_OB[self.OB_LOW_COL]
            else:
                entering_trigger_price = atf_support_resistance[self.SUPPORT_PRICE_KEY]

        # 4. 验证盈亏比
        entry_price = self.toDecimal(position[self.ENTRY_PRICE_KEY])
        profit_distance = abs(tp_price - entry_price)
        loss_distance = abs(entry_price - sl_price)

        if loss_distance > 0:
            actual_ratio = profit_distance / loss_distance
            if actual_ratio < min_profit_ratio:
                self.logger.info(
                    f"{symbol} : 盈亏比{actual_ratio:.2f} < {min_profit_ratio}，调整止盈价格"
                )
                # 调整止盈价格以满足最小盈亏比
                if pos_side == self.LONG_KEY:
                    tp_price = OPTools.safe_decimal_add(
                        entry_price,
                        OPTools.safe_decimal_multiply(loss_distance, min_profit_ratio),
                    )
                else:
                    tp_price = OPTools.safe_decimal_subtract(
                        entry_price,
                        OPTools.safe_decimal_multiply(loss_distance, min_profit_ratio),
                    )

        # 5. 使用自适应目标位查找器优化止盈价格
        use_adaptive_target = strategy.get("enable_adaptive_target", True)
        if use_adaptive_target:
            # 获取HTF数据用于自适应目标位分析
            htf_df = self.get_historical_klines_df(symbol=symbol, tf=htf)
            htf_struct = self.build_struct(
                symbol=symbol, data=htf_df, is_struct_body_break=open_body_break
            )
            
            original_tp_price = tp_price
            tp_price = self._find_adaptive_target_price(
                symbol=symbol,
                position=position,
                original_target=tp_price,
                htf_struct=htf_struct,
                atf_struct=atf_struct,
                strategy=strategy
            )
            
            if tp_price != original_tp_price:
                self.logger.info(
                    f"{symbol} : 自适应目标位调整 {original_tp_price:.{precision}f} -> {tp_price:.{precision}f}"
                )

        self.logger.info(
            f"{symbol} : 流动性抓取止盈止损 - 入场={entry_price:.{precision}f}, 止损={sl_price:.{precision}f}, 止盈={tp_price:.{precision}f}"
        )

        # 6. 设置止盈止损
        self.cancel_all_algo_orders(symbol=symbol, attachType=self.TP_KEY)
        has_tp = self.set_take_profit(
            symbol=symbol, position=position, tp_price=tp_price
        )

        self.cancel_all_algo_orders(symbol=symbol, attachType=self.SL_KEY)
        try:
            has_sl = self.set_stop_loss(
                symbol=symbol, position=position, sl_price=sl_price
            )
        except ValueError as e:
            self.logger.warning(f"{symbol} : 设置止损失败，使用默认方式: {e}")
            return self._init_default_SL_TP(symbol, position, strategy)

        # 7. 保存流动性目标和监控信息
        self.liquidity_targets[symbol] = liquidity_target
        self.entering_liquidity_tps[symbol] = entering_trigger_price

        return has_tp and has_sl

    def _identify_liquidity_target(
        self, symbol, atf_df, atf_struct, pos_side, strategy
    ):
        """识别等高等低流动性目标作为止盈位"""
        from core.smc.SMCLiquidity import SMCLiquidity
        
        # 获取配置参数
        atr_offset = strategy.get("liquidity_atr_offset", 0.1)
        min_profit_ratio = float(strategy.get("min_profit_ratio", 1.5))
        all_stop_loss_pct = float(strategy.get("all_stop_loss_pct", 2.0))
        max_search_depth = int(strategy.get("max_search_depth", 20))
        precision = self.get_precision_length(symbol)
        
        # 确定搜索类型
        point_type = "high" if pos_side == self.LONG_KEY else "low"
        
        # 使用 SMCLiquidity 查找等高等低点
        smc_liquidity = SMCLiquidity()
        result_df = smc_liquidity.identify_equal_points_in_range(
            data=atf_df,
            atr_offset=atr_offset,
            end_idx=-1,
            point_type=point_type,
            max_search_depth=max_search_depth,
        )
        
        # 打印等高等低流动性的关键信息用于调试
        if not result_df.empty:
            debug_columns = [
                smc_liquidity.TIMESTAMP_COL,
                smc_liquidity.EQUAL_POINTS_PRICE_COL,
                smc_liquidity.EQUAL_POINTS_TYPE_COL,
                smc_liquidity.EXTREME_VALUE_COL,
                smc_liquidity.ATR_TOLERANCE_COL,
                smc_liquidity.HAS_EQUAL_POINTS_COL,
            ]
            self.logger.debug(f"{symbol} : 等{point_type}点识别结果=\n{result_df[debug_columns].to_string()}")
        
        current_price = float(atf_df[self.CLOSE_COL].iloc[-1])
        # 计算最小利润要求 = 止损百分比 * 最小盈亏比
        min_profit_pct = all_stop_loss_pct * min_profit_ratio
        
        tp_price = None
        use_min_ratio = False
        
        if not result_df.empty:
            # 获取有等点的极值点价格
            equal_points = result_df[result_df[smc_liquidity.HAS_EQUAL_POINTS_COL] == True]
            
            if not equal_points.empty:
                if pos_side == self.LONG_KEY:
                    # 多头：查找当前价格上方的等高点
                    valid_targets = equal_points[equal_points[smc_liquidity.EXTREME_VALUE_COL] > current_price]
                    if not valid_targets.empty:
                        candidate_price = float(valid_targets[smc_liquidity.EXTREME_VALUE_COL].min())
                        profit_pct = ((candidate_price - current_price) / current_price) * 100
                        if profit_pct >= min_profit_pct:
                            tp_price = candidate_price
                            self.logger.info(f"{symbol} : 找到等高点止盈价格 {tp_price:.{precision}f} (当前价格={current_price:.{precision}f}, 利润空间={profit_pct:.2f}%)")
                        else:
                            self.logger.info(f"{symbol} : 等高点利润空间{profit_pct:.2f}% < 最小要求{min_profit_pct:.2f}%，使用最小盈亏比")
                            use_min_ratio = True
                else:
                    # 空头：查找当前价格下方的等低点
                    valid_targets = equal_points[equal_points[smc_liquidity.EXTREME_VALUE_COL] < current_price]
                    if not valid_targets.empty:
                        candidate_price = float(valid_targets[smc_liquidity.EXTREME_VALUE_COL].max())
                        profit_pct = ((current_price - candidate_price) / current_price) * 100
                        if profit_pct >= min_profit_pct:
                            tp_price = candidate_price
                            self.logger.info(f"{symbol} : 找到等低点止盈价格 {tp_price:.{precision}f} (当前价格={current_price:.{precision}f}, 利润空间={profit_pct:.2f}%)")
                        else:
                            self.logger.info(f"{symbol} : 等低点利润空间{profit_pct:.2f}% < 最小要求{min_profit_pct:.2f}%，使用最小盈亏比")
                            use_min_ratio = True
        
        # 未找到有效等点或利润不足，使用最小盈亏比计算止盈
        if tp_price is None:
            use_min_ratio = True
            if result_df.empty:
                self.logger.info(f"{symbol} : 未找到等{point_type}点，使用最小盈亏比")
        
        if use_min_ratio:
            if pos_side == self.LONG_KEY:
                tp_price = current_price * (1 + min_profit_pct / 100)
                self.logger.info(f"{symbol} : 按最小盈亏比设置止盈价格 {tp_price:.{precision}f} (当前价格={current_price:.{precision}f}, 利润空间={min_profit_pct:.2f}%)")
            else:
                tp_price = current_price * (1 - min_profit_pct / 100)
                self.logger.info(f"{symbol} : 按最小盈亏比设置止盈价格 {tp_price:.{precision}f} (当前价格={current_price:.{precision}f}, 利润空间={min_profit_pct:.2f}%)")
        
        return {
            "price": tp_price,
            "type": "equal_high" if pos_side == self.LONG_KEY else "equal_low",
            "trend": self.BULLISH_TREND if pos_side == self.LONG_KEY else self.BEARISH_TREND,
        }

    def _find_adaptive_target_price(
        self, symbol, position, original_target, htf_struct, atf_struct, strategy
    ):
        """
        使用 AdaptiveTargetFinder 查找自适应止盈目标位
        
        Args:
            symbol: 交易对
            position: 持仓信息
            original_target: 原始目标价格
            htf_struct: HTF结构数据
            atf_struct: ATF结构数据
            strategy: 策略配置
            
        Returns:
            自适应目标价格，如果未找到则返回原始目标价格
        """
        precision = self.get_precision_length(symbol)
        pos_side = position[self.SIDE_KEY]
        side = self.BUY_SIDE if pos_side == self.LONG_KEY else self.SELL_SIDE
        current_price = float(position[self.MARK_PRICE_KEY])
        
        # 构建 HTF 和 ATF 结果对象（简化版本）
        htf_latest = self.get_latest_struct(symbol=symbol, data=htf_struct)
        atf_latest = self.get_latest_struct(symbol=symbol, data=atf_struct)
        
        if not htf_latest or not atf_latest:
            self.logger.debug(f"{symbol} : 缺少结构数据，无法使用自适应目标位")
            return original_target
        
        # 创建简化的结果对象
        class SimpleResult:
            def __init__(result_self, struct_data):
                result_self.support_price = struct_data.get(self.STRUCT_LOW_COL)
                result_self.resistance_price = struct_data.get(self.STRUCT_HIGH_COL)
        
        htf_result = SimpleResult(htf_latest)
        atf_result = SimpleResult(atf_latest)
        
        # 调用自适应目标位查找器
        result = self.adaptive_target_finder.find_adaptive_target(
            symbol=symbol,
            current_price=current_price,
            original_target=float(original_target),
            side=side,
            htf_result=htf_result,
            atf_result=atf_result,
            precision=precision
        )
        
        if result.success:
            self.logger.info(
                f"{symbol} : 自适应目标位查找成功 - "
                f"原始目标={result.original_target_price:.{precision}f}, "
                f"新目标={result.new_target_price:.{precision}f}, "
                f"原因: {result.adjustment_reason}"
            )
            return self.toDecimal(result.new_target_price)
        else:
            self.logger.debug(
                f"{symbol} : 自适应目标位查找失败，使用原始目标 - {result.adjustment_reason}"
            )
            return original_target

    def _init_default_SL_TP(self, symbol, position, strategy):
        """使用默认百分比的止盈止损"""
        stop_loss_pct = OPTools.ensure_decimal(strategy.get("stop_loss_pct", 2))
        take_profile_pct = OPTools.ensure_decimal(strategy.get("take_profile_pct", 2))

        sl_price = self.calculate_sl_price_by_pct(
            symbol=symbol, position=position, sl_pct=stop_loss_pct
        )
        tp_price = self.calculate_tp_price_by_pct(
            symbol=symbol, position=position, tp_pct=take_profile_pct
        )

        self.cancel_all_algo_orders(symbol=symbol, attachType=self.TP_KEY)
        has_tp = self.set_take_profit(
            symbol=symbol, position=position, tp_price=tp_price
        )

        self.cancel_all_algo_orders(symbol=symbol, attachType=self.SL_KEY)
        has_sl = self.set_stop_loss(symbol=symbol, position=position, sl_price=sl_price)

        return has_tp and has_sl

    def _init_support_resistance_SL_TP(
        self, symbol, position, support_resistance, buffer_ticks
    ):
        """基于支撑阻力位的止盈止损"""
        tick_size = self.get_tick_size(symbol)
        price_offset = buffer_ticks * tick_size
        pos_side = position[self.SIDE_KEY]

        if pos_side == self.LONG_KEY:
            sl_price = self.toDecimal(
                support_resistance[self.SUPPORT_PRICE_KEY]
            ) - self.toDecimal(price_offset)
            tp_price = self.toDecimal(
                support_resistance[self.RESISTANCE_PRICE_KEY]
            ) + self.toDecimal(price_offset)
        else:
            sl_price = self.toDecimal(
                support_resistance[self.RESISTANCE_PRICE_KEY]
            ) + self.toDecimal(price_offset)
            tp_price = self.toDecimal(
                support_resistance[self.SUPPORT_PRICE_KEY]
            ) - self.toDecimal(price_offset)

        self.cancel_all_algo_orders(symbol=symbol, attachType=self.TP_KEY)
        has_tp = self.set_take_profit(
            symbol=symbol, position=position, tp_price=tp_price
        )

        self.cancel_all_algo_orders(symbol=symbol, attachType=self.SL_KEY)
        has_sl = self.set_stop_loss(symbol=symbol, position=position, sl_price=sl_price)

        return has_tp and has_sl

    def check_liquidity_grab_TP(
        self, symbol: str, position, tfs, strategy: dict
    ) -> bool:
        """
        流动性抓取特有的止盈监控
        监控价格是否触及流动性目标区域，并确认LTF结构反转
        """
        if (
            symbol not in self.entering_liquidity_tps
            or symbol not in self.liquidity_targets
        ):
            return False

        entering_trigger_price = self.entering_liquidity_tps[symbol]
        liquidity_target = self.liquidity_targets[symbol]
        market_price = position[self.MARK_PRICE_KEY]
        pos_side = position[self.SIDE_KEY]

        # 检查是否进入流动性监控区域
        if pos_side == self.LONG_KEY:
            in_monitoring_zone = market_price >= entering_trigger_price
        else:
            in_monitoring_zone = market_price <= entering_trigger_price

        if not in_monitoring_zone:
            return False

        precision = self.get_precision_length(symbol)
        self.logger.info(
            f"{symbol} : 进入流动性监控区域，当前价格{market_price:.{precision}f}，触发价格{entering_trigger_price:.{precision}f}"
        )

        # 检查是否接近流动性目标
        target_price = liquidity_target["price"]
        target_tolerance = 0.3  # 0.3%容差
        price_diff_pct = abs(market_price - target_price) / target_price * 100

        if price_diff_pct > target_tolerance:
            self.logger.debug(
                f"{symbol} : 距离流动性目标{target_price:.{precision}f}还有{price_diff_pct:.2f}%"
            )
            return False

        # 检查LTF结构反转确认
        open_body_break = strategy.get("open_body_break", True)
        etf = tfs[self.ETF_KEY]

        etf_df = self.get_historical_klines_df(symbol=symbol, tf=etf)
        etf_struct = self.build_struct(
            symbol=symbol, data=etf_df, is_struct_body_break=open_body_break
        )
        etf_latest_struct = self.get_latest_struct(
            symbol=symbol, data=etf_struct, is_struct_body_break=open_body_break
        )

        if not etf_latest_struct:
            self.logger.debug(f"{symbol} : LTF {etf} 未形成结构")
            return False

        etf_trend = etf_latest_struct[self.STRUCT_DIRECTION_COL]
        expected_trend = (
            self.BULLISH_TREND if pos_side == self.LONG_KEY else self.BEARISH_TREND
        )

        if etf_trend != expected_trend:
            self.logger.debug(
                f"{symbol} : LTF {etf} 结构{etf_trend}与预期{expected_trend}不符"
            )
            return False

        # 检查结构类型，CHOCH或BOS更可靠
        etf_struct_type = etf_latest_struct[self.STRUCT_COL]
        if etf_struct_type and ("CHOCH" in etf_struct_type or "BOS" in etf_struct_type):
            self.logger.info(
                f"{symbol} : LTF {etf} 结构反转确认 {etf_struct_type}，触发流动性止盈"
            )
            return True

        return False

    def trailing_SL_by_order_blocks(self, symbol: str, position, tfs, strategy: dict):
        """
        基于订单块的移动止损
        使用有效的订单块作为新的止损位
        """
        if symbol not in self.order_block_levels:
            return

        open_body_break = strategy.get("open_body_break", True)
        trailing_atr_multiplier = strategy.get("trailing_order_block_atr", 0.6)
        precision = self.get_precision_length(symbol)

        etf = tfs[self.ETF_KEY]
        pos_side = position[self.SIDE_KEY]
        market_price = position[self.MARK_PRICE_KEY]

        # 获取LTF数据寻找新的订单块
        etf_df = self.get_historical_klines_df_by_cache(symbol=symbol, tf=etf)
        etf_struct = self.build_struct(
            symbol=symbol, data=etf_df, is_struct_body_break=open_body_break
        )

        # 寻找有效的订单块
        side = (
            self.SELL_SIDE if pos_side == self.LONG_KEY else self.BUY_SIDE
        )  # 寻找反向订单块
        etf_OBs_df = self.find_OBs(
            symbol=symbol,
            struct=etf_struct,
            side=side,
            is_valid=True,
            is_struct_body_break=open_body_break,
            atr_multiplier=trailing_atr_multiplier,
        )

        if len(etf_OBs_df) == 0:
            self.logger.debug(f"{symbol} : 未找到有效的订单块用于移动止损")
            return

        # 过滤出符合条件的订单块
        if pos_side == self.LONG_KEY:
            # 多头：寻找当前价格下方的订单块作为新止损
            mask = etf_OBs_df[self.OB_HIGH_COL] < market_price
        else:
            # 空头：寻找当前价格上方的订单块作为新止损
            mask = etf_OBs_df[self.OB_LOW_COL] > market_price

        filtered_OBs = etf_OBs_df[mask]
        if len(filtered_OBs) == 0:
            return

        # 选择最新的订单块
        latest_OB = filtered_OBs.iloc[-1]

        # 验证订单块质量（大实体+小影线）
        if not self._validate_order_block_quality(latest_OB, strategy):
            return

        # 计算新的止损价格
        tick_size = self.get_tick_size(symbol)
        buffer_ticks = strategy.get("stop_loss_buffer_ticks", 2)
        price_offset = buffer_ticks * tick_size

        if pos_side == self.LONG_KEY:
            new_sl_price = self.toDecimal(latest_OB[self.OB_LOW_COL]) - self.toDecimal(
                price_offset
            )
        else:
            new_sl_price = self.toDecimal(latest_OB[self.OB_HIGH_COL]) + self.toDecimal(
                price_offset
            )

        # 验证新止损是否更优
        current_sl_price = self.get_stop_loss_price(symbol)
        should_update = False

        if current_sl_price is None:
            should_update = True
        elif pos_side == self.LONG_KEY and new_sl_price > current_sl_price:
            should_update = True
        elif pos_side == self.SHORT_KEY and new_sl_price < current_sl_price:
            should_update = True

        if should_update:
            old_sl = f"{current_sl_price:.{precision}f}" if current_sl_price else "未设置"
            self.logger.info(
                f"{symbol} : 基于订单块移动止损 {old_sl} -> {new_sl_price:.{precision}f}"
            )
            # 双止损模式下不要先取消，由 DualStopLossManager 管理
            if not (self.dual_sl_manager and self.dual_sl_manager.enabled):
                self.cancel_all_algo_orders(symbol=symbol, attachType=self.SL_KEY)
            self.set_stop_loss(symbol=symbol, position=position, sl_price=new_sl_price)
            self.order_block_levels[symbol] = (
                latest_OB[self.OB_LOW_COL]
                if pos_side == self.LONG_KEY
                else latest_OB[self.OB_HIGH_COL]
            )
        else:
            self.logger.debug(
                f"{symbol} : 新订单块止损{new_sl_price:.{precision}f}不优于当前止损{current_sl_price}"
            )

    def _validate_order_block_quality(self, order_block, strategy):
        """验证订单块质量（大实体+小影线标准）"""
        min_body_ratio = strategy.get("order_block_filter", {}).get(
            "min_body_ratio", 0.7
        )
        max_wick_ratio = strategy.get("order_block_filter", {}).get(
            "max_wick_ratio", 0.3
        )

        # 这里简化处理，实际应该根据订单块对应的K线数据来计算
        # 由于订单块数据中没有直接的实体和影线信息，暂时返回True
        # 在实际应用中可以通过订单块的timestamp找到对应的K线进行详细分析
        return True

    @override
    def process_pair(self, symbol: str, position, pair_config: dict) -> None:
        """处理单个交易对的流动性抓取策略"""
        precision = self.get_precision_length(symbol)

        liquidity_strategy = pair_config.get("liquidity_grab_strategy", {})
        stop_loss_pct = OPTools.ensure_decimal(pair_config.get("all_stop_loss_pct", 2))
        all_TP_SL_ratio = OPTools.ensure_decimal(
            pair_config.get("all_TP_SL_ratio", 1.5)
        )

        # 时间框架配置
        tfs = {
            self.HTF_KEY: str(liquidity_strategy.get("htf", "4h")),
            self.ATF_KEY: str(liquidity_strategy.get("atf", "15m")),
            self.ETF_KEY: str(liquidity_strategy.get("etf", "1m")),
        }

        htf, atf, etf = tfs[self.HTF_KEY], tfs[self.ATF_KEY], tfs[self.ETF_KEY]

        # 更新策略参数
        liquidity_strategy["stop_loss_pct"] = stop_loss_pct
        liquidity_strategy["take_profile_pct"] = OPTools.safe_decimal_multiply(
            stop_loss_pct, all_TP_SL_ratio
        )

        open_body_break = bool(liquidity_strategy.get("open_body_break", True))
        self.logger.info(
            f"{symbol} : 流动性抓取策略 {htf}|{atf}|{etf} open_body_break={open_body_break}"
        )

        # 1. 初始化流动性抓取止盈止损
        if symbol not in self.has_init_SL_TPs:
            has_pass = self.init_liquidity_grab_SL_TP(
                symbol, position, tfs, liquidity_strategy
            )

            if has_pass:
                self.has_init_SL_TPs[symbol] = True
                self.logger.info(f"{symbol} : 流动性抓取止盈止损初始化成功")
            else:
                self.logger.info(f"{symbol} : 流动性抓取止盈止损初始化失败")

        # 2. 基于订单块的移动止损
        enable_order_block_trailing = liquidity_strategy.get(
            "enable_order_block_trailing", True
        )
        if enable_order_block_trailing:
            self.logger.debug(f"{symbol} : 开启订单块移动止损...")
            self.trailing_SL_by_order_blocks(symbol, position, tfs, liquidity_strategy)
            # 打印当前止损订单信息
            if self.dual_sl_manager and self.dual_sl_manager.enabled:
                for i, order in enumerate(self.dual_sl_manager._get_active_orders(symbol), 1):
                    self.logger.debug(f"{symbol} : 止损单#{i} id={order.order_id} price={order.price} time={order.created_at.strftime('%H:%M:%S')}")

        # 3. 流动性目标止盈监控
        enable_liquidity_target_tp = liquidity_strategy.get(
            "enable_liquidity_target_tp", True
        )
        tp_structure_confirmation = liquidity_strategy.get(
            "tp_structure_confirmation", True
        )

        if enable_liquidity_target_tp and tp_structure_confirmation:
            if not self.check_liquidity_grab_TP(
                symbol, position, tfs, liquidity_strategy
            ):
                self.logger.debug(f"{symbol} : 未触发流动性止盈监控，等待...")
            else:
                order = self.close_position(symbol, position)
                if order:
                    self.logger.info(
                        f"{symbol} : 流动性目标达成，市价{position[self.MARK_PRICE_KEY]:.{precision}f}平仓"
                    )
